# frozen_string_literal: true

# Legende
# Les script a plusieurs parties indique que les bash (Batch dans l'app)
# peuvent s'initier avec differents types de source
# Le besoin est donc d'isoler la logique associee au modele actuel
# ne pas oublier d'indexer les champs simples

module Hermes
  # Status class
  class Status
    include Hermes::Concerns::CommonFieldsConcern
    field :reference_model_name, type: String
    field :status_label, type: String
    field :motive, type: String
    field :title, type: String
    field :object_id, type: String
    field :performer_id, type: BSON::ObjectId
    field :closed_by, type: BSON::ObjectId
    field :closed_at, type: DateTime
    field :closing, type: BSON::ObjectId

    belongs_to :applicant, class_name: 'Hermes::Applicant', index: true
    belongs_to :housing_request, class_name: 'Hermes::HousingRequest', index: true
    belongs_to :housing_offer, class_name: 'Hermes::HousingOffer', index: true
    belongs_to :housing_application, class_name: 'Hermes::HousingApplication', index: true
    belongs_to :entity, class_name: 'Hermes::Entity', index: true
    belongs_to :lessor, class_name: 'Hermes::Lessor', index: true
    belongs_to :required_document, class_name: 'Hermes::RequiredDocument', index: true

    before_create :before_create_actions
    after_create :after_create_actions

    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id

    common_indexes *%i[
      reference_model_name
      status_label
      motive
      title
      object_id
      performer_id
      closed_by
      closed_at
      closing
    ]

    CLASSES = [
      Hermes::Applicant,
      Hermes::HousingRequest,
      Hermes::HousingOffer,
      Hermes::HousingApplication,
      Hermes::Entity,
      Hermes::Lessor,
      Hermes::RequiredDocument
    ]

    def before_create_actions
      handle_reference_model_name
      check_if_closing
    end
    
    def after_create_actions
      update_object_status
      update_closing if closing.present?
    end

    def handle_reference_model_name
      CLASSES.any? do |model|
        if object = model.find_by(id: object_id)
          class_name = object.class.name
          send("#{class_name.demodulize.underscore}_id=", object.id)
          self.reference_model_name = class_name
        end
      end
    end

    def check_if_closing
      if (
        closing = Hermes::Status.only(:object_id, :status_label, :created_at)
                                .where(object_id: object_id, status_label: status_label)
                                .order(created_at: :desc)
                                .limit(1)
                                .first
      )
        self.closing = closing.id
      end
    end

    def update_object_status
      if (reference_model_name && instance = reference_model_name.to_s.constantize.find_by(id: object_id))
        if instance[status_label].kind_of?(Array)
          instance["old_#{status_label}"] = instance[status_label]

          status_info = {
            title: title, 
            id: id.to_s, 
            updated_at: created_at, 
            performer_id: performer_id.to_s,
            closing: closing.to_s
          }

  
          if prev = instance[status_label].detect {|e| e[:performer_id] == performer_id.to_s}
            instance[status_label] = instance[status_label].map { |e| e[:performer_id] == performer_id.to_s ? status_info : e }
          else
            instance[status_label] = instance[status_label] + [status_info]
          end 

          else
            instance[status_label] = title
            instance["#{status_label}_id"] = id
            instance["old_#{status_label}"] = Hermes::Status.find(closing).title if closing
          end
          instance["#{status_label}_updated_at"] = created_at
          instance.save
      end
    end

    def update_closing
      c = Hermes::Status.find(closing)
      c.closed_at = DateTime.current
      c.closed_by = id
      c.save!
    end
  end
end
