require 'rails_helper'
module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  MUTATE_PARAMS = {
    status_label: String,
    motive: String,
    title: String
  }

  ACCEPTABLE_PARAMS = %w[
    model_name
    status_label
    motive
    title
    object_id
    performer_id
    closed_by_id
    closed_at
    closing_id
    applicant_id
    housing_request_id
    housing_offer_id
    housing_application_id
    entity_id
    lessor_id
  ]

  describe Api::Dmo::StatusesController, :type => :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper
    let!(:responsible) {create(:user, roles: ['Administrator'])}
    let!(:user) {create(:user)}
    let!(:housing_offer){create(:housing_offer)}
    #TESTS BASIQUES D'API

    describe 'POST /statuses' do
      describe 'Create status with minimal valid datas' do
        before :each do
          authenticated_header(request, responsible)
          post 'create', {
              data: {
                  attributes: {
                      object_id: housing_offer.id.to_s,
                  }
              }
          }
        end
        it 'returns status code 201' do
          expect(response).to have_http_status(201)
          expect(json['data']['attributes']['housing_offer_id']).to eq(housing_offer.id.to_s)
        end
      end
      MUTATE_PARAMS.each do |param, type|
        describe 'post request with ' + param.to_s + ' (' + type.to_s + ') is valid' do
          let(:params) do
            attr = {}
            attr[param] = get_typed_value(type)
            attr
          end
          before :each do
            params['user_id'] = user.id.to_s
            authenticated_header(request, responsible)
            post 'create', {
                data: {
                    attributes: {
                        object_id: housing_offer.id.to_s,
                    }
                }
            }
          end
          it 'returns status code 201 and the value appears in json' do
            expect(response).to have_http_status(201)
            if (type == 'Date' || type == 'DateTime')
              expect(json['data']['attributes'][param.to_s]).not_to be_empty
              expect(json['data']['attributes'][param.to_s]).not_to eq('')
            else
              expect(json['data']['attributes'][param.to_s]).to eq(get_typed_value(type))
            end
          end
        end
      end
    end

    describe 'GET /statuses' do
      describe 'GET /statuses' do
        let!(:status1) {create(:status, object_id: housing_offer.id.to_s)}
        let!(:status3) {create(:status, object_id: housing_offer.id.to_s)}
        let!(:status2) {create(:status, object_id: housing_offer.id.to_s)}
        before :each do
          authenticated_header(request, responsible)
          get 'index'
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(3)
          expect(json['meta']['pagination']).to be_present
          expect(json['meta']['pagination']['total_objects']).to eq(3)
        end
      end

      describe 'GET /statuses with batch of ids' do
        let!(:status1) {create(:status, object_id: housing_offer.id.to_s)}
        let!(:status3){create(:status, object_id: housing_offer.id.to_s)}
        let!(:status2){create(:status, object_id: housing_offer.id.to_s)}
        before :each do
          authenticated_header(request, responsible)
          get 'index', {ids: [status1.id.to_s, status2.id.to_s]}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(2)
        end
      end

      ACCEPTABLE_PARAMS.each do |acceptable_param|
        type = MUTATE_PARAMS[acceptable_param.to_sym] || MUTATE_PARAMS[acceptable_param.to_s]
        if type
          describe 'GET /statuses with sort on field ' + acceptable_param + ' works ' do
            let!(:status3) {create(:status, acceptable_param => get_typed_value(type, 2), object_id: housing_offer.id.to_s)}
            let!(:status1) {create(:status, acceptable_param => get_typed_value(type, 0), object_id: housing_offer.id.to_s)}
            let!(:status2) {create(:status, acceptable_param => get_typed_value(type, 1), object_id: housing_offer.id.to_s)}
            before :each do
              authenticated_header(request, responsible)
              get 'index', {sort: {acceptable_param => 1}}
            end
            it 'returns sorted results' do
              expect(response.status).to eq(200)
              if type == 'DateTime'
                status_attribute = json['data'].map {|unit| DateTime.parse(unit['attributes'][acceptable_param])}
                expect(status_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
              elsif type == 'Date'
                status_attribute = json['data'].map {|unit| Date.parse(unit['attributes'][acceptable_param])}
                expect(status_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
              else
                status_attribute = json['data'].map {|unit| unit['attributes'][acceptable_param]}
                expect(status_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
              end
            end
          end
        end
      end
      describe 'GET /statuses with operators' do
        let!(:status1) {create(:status, motive: 'acceuillant', object_id: housing_offer.id.to_s)}
        let!(:status3) {create(:status, motive: 'Super', object_id: housing_offer.id.to_s)}
        let!(:status2) {create(:status, motive: 'bel appart', object_id: housing_offer.id.to_s)}

        describe 'with $in' do
          before :each do
            authenticated_header(request, responsible)
            get 'index', {motive: {'$in': ['acceuillant', 'flamand']}}
          end
          it 'returns results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(1)
          end
        end
        describe 'with $ne' do
          before :each do
            authenticated_header(request, responsible)
            get 'index', {motive: {'$ne': 'acceuillant'}}
          end
          it 'returns sorted results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(2)
          end
        end
        describe 'with $regex' do
          before :each do
            authenticated_header(request, responsible)
            get 'index', {motive: {'$regex': /^S/}}
          end
          it 'returns sorted results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(1)
          end
        end
      end
    end

    describe 'GET /statuses/:id' do
      describe 'verif it work' do
        let!(:status1) {create(:status, motive: 'acceuillant', object_id: housing_offer.id.to_s)}
        before :each do
          authenticated_header(request, responsible)
          get 'show', {id: status1.id}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
        end
      end
      describe 'verif it work not work with a non authorized user' do
        let!(:status1) {create(:status, object_id: housing_offer.id.to_s)}

        before :each do
          authenticated_header(request, user)
          get 'show', {id: status1.id}
        end
        it 'returns status code 404' do
          expect(response).to have_http_status(404)
        end
      end
    end

    describe 'PATCH /statuses/:id' do
      status_params = {}
      MUTATE_PARAMS.each do |param, type|
        status_params[param] = get_typed_value(type)
      end

      let!(:status) {create(:status, **status_params, object_id: housing_offer.id.to_s)}
      MUTATE_PARAMS.each do |param, type|
        describe 'patch request with ' + param.to_s + ' ' + type.to_s + 'is valid' do
          let(:params) do
            attr = {}
            attr[param] = get_typed_value(type, 2)
            attr
          end
          before :each do

            authenticated_header(request, responsible)
            patch 'update', {
                id: status.id.to_s,
                data: {
                    attributes: params
                }
            }
          end
          it 'returns status code 200 and the value appears in json' do
            expect(response).to have_http_status(200)
            if (type == 'Date' || type == 'DateTime')

              expect(json['data']['attributes'][param.to_s]).not_to be_empty
              expect(json['data']['attributes'][param.to_s]).not_to eq('')
            else
              expect(json['data']['attributes'][param.to_s]).to eq(get_typed_value(type, 2))
            end
          end
        end
      end

      describe 'DELETE /statuses/:id' do
        let!(:status) {create(:status, object_id: housing_offer.id.to_s, user_id: responsible.id.to_s)}
        before :each do
          authenticated_header(request, responsible)

          delete 'destroy', {
              id: status.id.to_s
          }
        end
        it 'returns status code 204' do
          expect(response).to have_http_status(204)
        end
      end
    end

    describe 'GET /statuses/applications_history' do
      let!(:demandeur) { create :user, roles: ['demandeur'] }
      let!(:application) { create :active_application, user_id: demandeur.id }
      let!(:rand_application) { create :active_application }
      let!(:status_1) { create :status, object_id: application.id, status_label: 'status' , title: 'ANNULEE' }
      let!(:status_2) { create :status, object_id: application.id, status_label: 'status' , title: 'INSCRIPTION CAL' }
      let!(:status_3) { create :status, object_id: application.id, status_label: 'status' , title: 'TRANSMIS BAILLEUR' }
      let!(:statuses) { create_list :status, 4, object_id: rand_application.id, status_label: 'status' , title: 'NON RETENUE' }

      context 'when demandeur' do
        let(:expected_ids) { json['data'].map { |h| h['id'] } }

        before :each do
          status_1.set(updated_at: Time.zone.now - 3.days)
          status_2.set(updated_at: Time.zone.now + 5.days)

          authenticated_header(request, demandeur)
          get 'applications_history'
        end
        it 'succeeds' do
          expect(response).to have_http_status 200
        end

        it 'returns only the user applications_history' do
          expect(json['data'].length).to eq 3
          expect(expected_ids).to eq [status_1.id.to_s, status_2.id.to_s, status_3.id.to_s]
        end

        context 'when start_date' do
          it 'filters by updated_at' do
            get 'applications_history', { start_date: Time.zone.now - 2.days }
            expect(json['data'].length).to eq 2
            expect(expected_ids).to eq [status_2.id.to_s, status_3.id.to_s]
          end
        end

        context 'when end_date' do
          it 'filters by updated_at' do
            get 'applications_history', { end_date: Time.zone.now + 4.days }
            expect(json['data'].length).to eq 2
            expect(expected_ids).to eq [status_1.id.to_s, status_3.id.to_s]
          end
        end

        context 'when both dates' do
          it 'filters by updated_at' do
            get 'applications_history', { start_date: Time.zone.now - 2.days, end_date: Time.zone.now + 4.days }
            expect(json['data'].length).to eq 1
            expect(expected_ids).to eq [status_3.id.to_s]
          end
        end
      end

      context 'when other role' do
        before :each do
          authenticated_header(request, [responsible, user].sample)
          get 'applications_history'
        end
        it 'returns unauthorized error' do
          expect(response).to have_http_status 401
        end
      end
    end
  end

  require_relative '../../swagger_helper'

  describe Hermes::Api::Dmo::StatusesController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    TAGS_API_DMO_STATUSES = 'API_DMO_STATUSES'
    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes

    context 'as admin' do
      let!(:admin) {create(:administrator)}
      let!(:housing_offer){create(:housing_offer)}

      path '/api/dmo/statuses' do
        post 'Create status' do
          tags TAGS_API_DMO_STATUSES
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          json_params = {}
          json_params[:object_id] = {type: :string}
          json_params[:performer_id] = {type: :string}
          json_params[:status_label] = {type: :string}
              json_params[:motive] = {type: :string}
              json_params[:title] = {type: :string}
          parameter name: :params, in: :body, schema: jsonify_params(json_params)

          response '201', 'status' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:params) do
              jsonapi_attributes(
              {object_id: housing_offer.id.to_s}

              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'statuses'
            end
          end
        end
      end

      path '/api/dmo/statuses' do
        get 'Retrieves all statuses' do
          tags TAGS_API_DMO_STATUSES
          produces 'application/json'

          parameter name: 'ids[]', in: :query, type: :array, items: { type: :string }, collectionFormat: :multi, required: false
          ACCEPTABLE_PARAMS.each do |acceptable_param|
            type = ''
            if MUTATE_PARAMS.include?(acceptable_param)
              type = nice_type(MUTATE_PARAMS[acceptable_param])
            end
            parameter name: acceptable_param.to_s, in: :query, type: type, required: false
          end
          # VALID_OPERATORS.each do |valid_operator|
          #  parameter( name: valid_operator.to_s, in: :query,type: :object, required: false)
          # end

          security [Bearer: {}]
          paginator
          response '200', 'statuses found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let!(:statuses) {create_list(:status, 5, object_id: housing_offer.id.to_s,user_id: admin.id.to_s)}
            include_context 'valid jsonapi collection' do
              let(:expected_collection) {statuses}
            end
          end
        end
      end

      path '/api/dmo/statuses/{id}' do
        get 'Retrieves a single status by id' do
          tags TAGS_API_DMO_STATUSES
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '200', 'status found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:status) {create(:status, object_id: housing_offer.id.to_s,user_id: admin.id.to_s)}
            let(:id) {status.id}

            include_context 'with integration test'
            include_context 'valid jsonapi ressource' do
              let(:expected_ressource) {status}
            end
          end
        end
      end

      path '/api/dmo/statuses/{id}' do
        put 'edit status' do
          tags TAGS_API_DMO_STATUSES
          produces 'application/json'
          consumes 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = {type: nice_type(val)}
          end
          json_params[:lessor_id] = {type: :string}
          parameter name: :params, in: :body, schema: jsonify_params(json_params)

          response '200', 'update status' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:status) {create(:status, object_id: housing_offer.id.to_s)}
            let(:id) {status.id}
            let(:params) do
              jsonapi_attributes(
                  id: id,
                  user_id: admin.id.to_s
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'statuses'
            end
          end
        end
      end

      path '/api/dmo/statuses/{id}' do
        delete 'Delete a single status by id' do
          tags TAGS_API_DMO_STATUSES
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '204', 'status deleted' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:id) {create(:status, object_id: housing_offer.id.to_s,user_id: admin.id.to_s).id}
            run_test!
          end
        end
      end
    end

    context 'as demandeur' do
      let!(:demandeur) { create :user, roles: ['demandeur'] }
      let!(:housing_application) { create :active_application, user_id: demandeur.id }

      path '/api/dmo/statuses/applications_history' do
        get 'Retrieves all user\'s applications statuses' do
          tags TAGS_API_DMO_STATUSES
          produces 'application/json'
          parameter name: 'start_date', in: :query, type: :string, required: false
          parameter name: 'end_date', in: :query, type: :string, required: false
          security [Bearer: {}]
          paginator
          response '200', 'statuses found' do
            let(:Authorization) {"Bearer #{token_for(demandeur)}"}
            let!(:statuses) { create_list :status, 5, object_id: housing_application.id , status_label: 'status', title: 'EN ATTENTE'}

            include_context 'with integration test'
            include_context 'valid jsonapi collection' do
              let(:expected_collection) { statuses }
            end
          end
        end
      end
    end
  end
end
