require 'rails_helper'

RSpec.describe Hermes::EntityReservation, type: :model do
  context 'with basic setup' do
    let!(:entity_1) do
      create(
        :entity,
        title: 'Entreprise SA',
        id_rex: '1234'
      )
    end
  
    let!(:entity_2) do
      create(
        :entity,
        title: 'Test SAS',
        id_rex: '2302'
      )
    end
  
    let!(:housing_offer_1) do
      create(
        :housing_offer,
        offer_status: 'PUBLIE',
      )
    end
  
    let!(:entity_reservation_1) do
      create(
        :entity_reservation,
        entity: entity_1,
        housing_offer: housing_offer_1,
        from:DateTime.now() - 30.days,
        to: DateTime.now() + 30.days,
      )
    end
  
    let!(:entity_reservation_2) do
      create(
        :entity_reservation,
        entity: entity_2,
        housing_offer: housing_offer_1,
        from: DateTime.now() - 30.days,
        to: DateTime.now() + 30.days,
      )
    end
    
    describe 'on creation' do
      it 'attributes are setup properly' do
        expect(entity_reservation_1.reload.is_active).to eq(true)
        expect(housing_offer_1.reload.reserved).to eq(true)
        expect(housing_offer_1.reload.active_reservations).to eq([entity_1.id.to_s, entity_2.id.to_s])
        expect(housing_offer_1.reload.active_reservations_rex_ids).to eq([entity_1.id_rex.to_s, entity_2.id_rex.to_s])
        expect(housing_offer_1.reload.eligibility_query.json_to_h[:$and][0]['entity_id']['$in']).to eq(housing_offer_1.reload.active_reservations)
        expect(Hermes::Status.all().length).to eq(3)
        expect(housing_offer_1.reload.entity_status[0][:title]).to eq("Réservé par Entreprise SA")
        expect(housing_offer_1.reload.entity_status[1][:title]).to eq("Réservé par Test SAS")
      end
    end
    
    describe 'on non active reservation' do     
      it 'on past date' do
        entity_reservation_1.to = DateTime.now() - 2.days
        entity_reservation_1.save
        expect(entity_reservation_1.reload.is_active).to eq(false)
        expect(housing_offer_1.reload.active_reservations.length).to eq(1)
        expect(housing_offer_1.reload.active_reservations_rex_ids.length).to eq(1)
        expect(Hermes::Status.all().length).to eq(4)
        expect(housing_offer_1.reload.entity_status[0][:title]).to eq("Fin de réservation pour Entreprise SA")
      end
    end
    
    describe 'on update eligibility' do
      it 'attributes are setup properly' do
        entity_reservation_1.update_activity(DateTime.now() + 31.days)
        expect(entity_reservation_1.reload.is_active).to eq(false)
        expect(housing_offer_1.reload.active_reservations.length).to eq(1)
        expect(housing_offer_1.reload.active_reservations_rex_ids.length).to eq(1)
        expect(Hermes::Status.all().length).to eq(4)
        expect(housing_offer_1.reload.entity_status[0][:title]).to eq("Fin de réservation pour Entreprise SA")
      end
    end
    
    describe 'remove reservation' do
      it 'attributes are setup properly' do
        entity_reservation_2.to =  DateTime.now() - 2.days
        entity_reservation_2.save
        expect(entity_reservation_2.reload.is_active).to eq(false)
        expect(housing_offer_1.reload.reserved).to eq(true)
        expect(Hermes::Status.all().length).to eq(4)
        expect(housing_offer_1.reload.entity_status[1][:title]).to eq("Fin de réservation pour Test SAS")
        expect(housing_offer_1.reload.active_reservations).to eq([entity_1.id.to_s])
        expect(housing_offer_1.reload.active_reservations_rex_ids).to eq([entity_1.id_rex.to_s])
      end
    end
    
  end
end
