require 'rails_helper'

RSpec.describe Hermes::HousingOffer, type: :model do
  it 'has a valid factory' do
    expect(FactoryBot.build(:housing_offer)).to be_valid
  end

  describe 'validate save side entities relationships effect' do
    let!(:entity1) { create(:entity, siret: 'test1000000000') }
    let!(:entity2) { create(:entity, siret: 'test2000000000') }
    let!(:entity3) { create(:entity, siret: 'test3000000000') }

    let!(:offer) do
      create(
        :housing_offer,
        entities_siret: ['test1000000000', 'test3000000000']
      )
    end

    it 'should has valid relationships' do
      expect(Hermes::HousingEntityOffer.where(housing_offer: offer).count).to be 2
      expect(offer.entity_ids).to include(entity1.id).and(include(entity3.id))
      expect(offer.readers).to include("g_#{entity1.supervisor_group.id}").and(include("g_#{entity3.supervisor_group.id}"))
      expect(offer.writers).to include("g_#{entity1.manager_group.id}").and(include("g_#{entity3.manager_group.id}"))
    end

    describe 'changing entities_siret' do
      before :each do
        offer.entities_siret = ['test1000000000', 'test2000000000']
        offer.save
      end

      it 'should has valid relationships' do
        expect(Hermes::HousingEntityOffer.where(housing_offer: offer).count).to be 2
        expect(Hermes::HousingEntityOffer.unscoped.where(housing_offer: offer).count).to be 3
        expect(offer.reload.entity_ids).to include(entity1.id).and(include(entity2.id))
        expect(offer.reload.readers).not_to include("g_#{entity3.supervisor_group.id}")
        expect(offer.reload.writers).not_to include("g_#{entity3.manager_group.id}")
        expect(offer.reload.readers).to include("g_#{entity1.supervisor_group.id}").and(include("g_#{entity2.supervisor_group.id}"))
        expect(offer.reload.writers).to include("g_#{entity1.manager_group.id}").and(include("g_#{entity2.manager_group.id}"))
      end
    end
  end

  describe 'validate save side charges effect' do
    let!(:basic_housing_offer) do
      create(
        :housing_offer,
        rental_charges: 24.5,
        parking_charges: 12.3,
        rent_amount: 78.0,
        rent_with_charges: 78.0 + 12.3 + 24.5,
        typology: 'T2',
        heating_type: 'Individuel électrique'
      )
    end

    it 'put referent' do
      expect(basic_housing_offer.total_charges).to eq(108.52)
    end
  end

  describe 'without heating charges' do
    let!(:basic_housing_offer) do
      create(
        :housing_offer,
        rental_charges: 24.5,
        parking_charges: 12.3,
        rent_amount: 78.0,
        rent_with_charges: 78.0 + 12.3 + 24.5,
        typology: 'T1',
        heating_type: 'Collectif'
      )
    end

    it 'put referent' do
      expect(basic_housing_offer.total_charges).to eq(36.8)
    end
  end

  describe 'validate save is_mad side effect' do
    let!(:housing_offer1) do
      create(
        :housing_offer,
        lessor_status: 'PLACE',
        offer_status: "EN ATTENTE COMMERCIALISATION"
      )
    end

    let!(:housing_offer2) do
      create(
        :housing_offer,
        lessor_status: 'RETIRE'
      )
    end

    let!(:housing_offer3) do
      create(
        :housing_offer,
        offer_status: 'RETIRE PAR LE BAILLEUR'
      )
    end

    let!(:housing_offer4) do
      create(
        :housing_offer,
        offer_status: 'LOGEMENT PLACE'
      )
    end

    it 'put referent' do
      expect((1..4).map { |i| send("housing_offer#{i}").is_mad }).to all(be false)
    end

    describe 'when status gets changed to mad situation' do
      before :each do
        housing_offer1.update(lessor_status: 'EN ATTENTE PUBLICATION')
      end

      it 'should has the right status' do
        expect(housing_offer1.reload.lessor_status).to eq 'EN ATTENTE PUBLICATION'
      end

      it 'should has updated correctly' do
        offer1 = housing_offer1.reload
        expect(offer1.is_mad).to be true
        expect(offer1.is_mad_updated_at).to_not eq offer1.created_at
      end
    end
  end

  describe 'with adequate data for score' do
    #let!(:admin1) { create(:administrator) }

    let!(:threshold1) do
      create(
        :threshold,
        type: 'PLA LM',
        thresholds_map: {
          'Paris et communes limitrophes': {
            'thresholds_list': [
              13_050,
              21_272,
              27_883,
              30_521,
              36_307,
              40_859
            ],
            'floors_list': [
              3_050,
              11_272,
              17_883,
              20_521,
              26_307,
              30_859
            ],
            'another_person': 4552
          },
          'IDF hors Paris et Communes limitrophes': {
            'thresholds_list': [
              13_050,
              21_272,
              25_569,
              28_075,
              33_238,
              37_401
            ],
            'floors_list': [
              3_050,
              11_272,
              17_883,
              20_521,
              26_307,
              30_859
            ],
            'another_person': 4166
          },
          'Autres régions': {
            'thresholds_list': [
              11_342,
              16_525,
              19_872,
              22_111,
              25_870,
              29_155
            ],
            'floors_list': [
              3_050,
              11_272,
              17_883,
              20_521,
              26_307,
              30_859
            ],
            'another_person': 3252
          }
        }
      )
    end

    let(:entity1) { create(:entity) }

    let!(:user1) { create(:user) }
    let!(:applicant1) do
      create(
        :applicant,
        birth_date: nil,
        sne_profile_creation_date: '2016-09-24',
        main_user_id: user1.id,
        amount_year_minus_two: 9787.24,
        entity: entity1,
        monthly_incomes: {
          'Salaire': 1045.23,
          'APL': 321.62
        }
      )
    end
    let!(:request1) do
      create(
        :housing_request,
        is_active: true,
        main_applicant: applicant1,
        seeked_districts: ['34507'],
        seeked_departments: ['34'],
        entity_validated: true,
        priority_points_indexes: [30]
      )
    end
    let!(:nud1) do
      create(
        :nud,
        housing_request: request1,
        department: '34',
        creation_date: DateTime.now - 50.months
      )
    end

    let!(:user2) { create(:user) }
    let!(:applicant2) do
      create(
        :applicant,
        birth_date: nil,
        sne_profile_creation_date: '2017-09-24',
        main_user_id: user1.id,
        amount_year_minus_two: 9787.24,
        entity: entity1,
        monthly_incomes: {
          'Salaire': 1045.23,
          'APL': 321.62
        }
      )
    end
    let!(:request2) do
      create(
        :housing_request,
        is_active: true,
        main_applicant: applicant2,
        seeked_districts: ['34507'],
        seeked_departments: ['34'],
        entity_validated: true,
        priority_points_indexes: [40]
      )
    end
    let!(:nud2) do
      create(
        :nud,
        housing_request: request2,
        department: '34',
        creation_date: DateTime.now - 40.months
      )
    end

    let!(:user3) { create(:user) }
    let!(:applicant3) do
      create(
        :applicant,
        birth_date: nil,
        sne_profile_creation_date: '2018-09-24',
        main_user_id: user3.id,
        amount_year_minus_two: 9787.24,
        entity: entity1,
        monthly_incomes: {
          'Salaire': 1045.23,
          'APL': 321.62
        }
      )
    end
    let!(:request3) do
      create(
        :housing_request,
        is_active: true,
        main_applicant: applicant3,
        seeked_districts: ['34507'],
        seeked_departments: ['34'],
        entity_validated: true,
        priority_points_indexes: [30]
      )
    end
    let!(:nud3) do
      create(
        :nud,
        housing_request: request3,
        department: '34',
        creation_date: DateTime.now - 30.months
      )
    end

    let!(:user4) { create(:user) }
    let!(:applicant4) do
      create(
        :applicant,
        birth_date: nil,
        sne_profile_creation_date: '2015-09-24',
        main_user_id: user4.id,
        amount_year_minus_two: 9787.24,
        entity: entity1,
        monthly_incomes: {
          'Salaire': 1045.23,
          'APL': 321.62
        }
      )
    end
    let!(:request4) do
      create(
        :housing_request,
        is_active: true,
        main_applicant: applicant4,
        seeked_districts: ['34507'],
        seeked_departments: ['34'],
        entity_validated: true,
        priority_points_indexes: [40]
      )
    end
    let!(:nud4) do
      create(
        :nud,
        housing_request: request4,
        department: '34',
        creation_date: DateTime.now - 20.months
      )
    end

    let!(:offer1) do
      create(
        :housing_offer,
        typology: 'T2',
        rent_with_charges: 234.32 + 42.15 + 12.71,
        rent_amount: 234.32,
        rental_charges: 42.15,
        parking_charges: 12.71,
        surface: 29,
        offer_status: 'LOGEMENT PUBLIE',
        threshold_type: 'PLA LM',
        department: '34',
        min_effort_rate: 0.0,
        threshold_subtype: 'IDF hors Paris et Communes limitrophes'
      )
    end

    let!(:offer2) do
      create(
        :housing_offer,
        typology: 'T2',
        rent_with_charges: 434.32 + 53.12 + 22.43,
        rent_amount: 434.32,
        rental_charges: 53.12,
        parking_charges: 22.43,
        surface: 34,
        offer_status: 'LOGEMENT PUBLIE',
        threshold_type: 'PLA LM',
        department: '34',
        min_effort_rate: 0.0,
        threshold_subtype: 'Autres régions'
      )
    end


    before do
      [request1, request2, request3, request4].each do |hr|
        hr.validate_all_required_document
      end
      drain_all_jobs
    end

    describe 'handles queued jobs' do
      it 'should have jobs and right quantity of scores' do
        expect(enqueued_jobs).to be_present
        expect(enqueued_jobs.size).to be > 0
        perform_enqueued_jobs do
          enqueued_jobs.each do |job|
            VCR.use_cassette("hermes_geo_jobs", :match_requests_on => [:method, :uri]) do
              job[:job].perform_now(*deserialize_argument(job[:args]))
            end
          end
        end

        scores = Hermes::Score.where(housing_offer_id: offer1.id)
        expect(Hermes::HousingOffer.count).to be 2
        expect(Hermes::Score.unscoped.count).to be 8
        score_ids = scores.pluck(:id)

        expect(scores.count).to eq(4)
        applicant1_score = scores.find_by(applicant: applicant1)
        applicant2_score = scores.find_by(applicant: applicant2)
        applicant3_score = scores.find_by(applicant: applicant3)
        applicant4_score = scores.find_by(applicant: applicant4)

        expect(applicant1_score.profile_epci_score).to be 30 + 240 + 50
        expect(applicant1_score.eligible_position).to be 1
        expect(applicant1_score.applicated_position).to be 1

        expect(applicant2_score.profile_epci_score).to be 40 + 240 + 40 
        expect(applicant2_score.eligible_position).to be 2
        expect(applicant2_score.applicated_position).to be 1

        expect(applicant3_score.profile_epci_score).to be 30 + 240  + 30
        expect(applicant3_score.eligible_position).to be 3
        expect(applicant3_score.applicated_position).to be 1

        expect(applicant4_score.profile_epci_score).to be 40 + 240 + 20
        expect(applicant4_score.eligible_position).to be 4
        expect(applicant4_score.applicated_position).to be 1

        expect(offer1.reload.eligible_requests_nb).to be 4
        expect(offer1.reload.floor_category_1).to be 3050.0
        
      end
    end
  end

  describe 'Update realized_by_typologies of linked lessor_objective' do
    let!(:linked_lessor) { create(:lessor) }

    let!(:basic_housing_offer) do
      create(
        :housing_offer,
        lessor: linked_lessor,
        lessor_status: 'not rented',
        typology: 'T2',
        lessor_status_updated_at: DateTime.current # à supprimer une fois les statuts mis à jour
      )
    end

    let!(:linked_lessor_objective) do
      create(
        :lessor_objective,
        activation_date: DateTime.current,
        lessor: linked_lessor,
        target_by_typologies: {:all =>100},
      )
    end

    it 'setting up' do
      linked_lessor_objective.reload
      expect(linked_lessor_objective.target_by_typologies).to(eq({'all'=>100}))
      expect(linked_lessor_objective.present_typologies).to(eq(['all']))
      expect(linked_lessor_objective.realized_by_typologies).to(eq({'all'=>0}))
    end
  end

  describe 'Realized calculation work at objective creation' do
    let!(:linked_lessor) { create(:lessor) }

    let!(:basic_housing_offer1) do
      create(
        :housing_offer,
        lessor: linked_lessor,
        typology: 'T2',
      )
    end

    let!(:status1) do
      create(
        :status,
        object_id: basic_housing_offer1.id.to_s,
        status_label: 'lessor_status',
        title: 'PLACE'
      )
    end

    let!(:basic_housing_offer2) do
      create(
        :housing_offer,
        lessor: linked_lessor,
        typology: 'T2',
      )
    end

    let!(:status2) do
      create(
        :status,
        object_id: basic_housing_offer2.id.to_s,
        status_label: 'lessor_status',
        title: 'PLACE',
      )
    end

    let!(:linked_lessor_objective) do
      create(
        :lessor_objective,
        activation_date: DateTime.current,
        lessor: linked_lessor,
        target_by_typologies: {:T2 =>100},
      )
    end

    it 'count when offer placed' do
      expect(linked_lessor_objective.target_by_typologies).to(eq({:T2=>100}))
      expect(linked_lessor_objective.present_typologies).to(eq([:T2]))
      expect(linked_lessor_objective.realized_by_typologies).to(eq({:T2=>2}))
    end
  end

  describe 'Realized calculation work at objective creation within old env' do
    before :each do
      config = Setting.find_or_create_by(key: 'config', env: Rails.env)
      config.set(
        'values.transition_handler'.to_sym => 'old',
        'values.offer_family_slug'.to_sym => 'dev_housing_offers'
      )
      expect(config.reload.values.keys).to include('transition_handler').and(include('offer_family_slug'))
      expect(config.reload.values['transition_handler']).to eq('old')
      expect(config.reload.values['offer_family_slug']).to eq('dev_housing_offers')
    end

    let!(:linked_lessor) { create(:lessor) }

    let!(:basic_housing_offer1) do
      config = Setting.find_or_create_by(key: 'config', env: Rails.env)
      config.set(
        'values.transition_handler'.to_sym => 'old',
        'values.offer_family_slug'.to_sym => 'dev_housing_offers'
      )
      expect(config.reload.values.keys).to include('transition_handler').and(include('offer_family_slug'))
      expect(config.reload.values['transition_handler']).to eq('old')
      expect(config.reload.values['offer_family_slug']).to eq('dev_housing_offers')
      create(
        :housing_offer,
        lessor: linked_lessor,
        lessor_status: 'PLACE',
        typology: 'T2',
        lessor_status_updated_at: DateTime.current # TODO: à supprimer une fois les statuts mis à jour
      )
    end

    let!(:basic_housing_offer2) do
      config = Setting.find_or_create_by(key: 'config', env: Rails.env)
      config.set(
        'values.transition_handler'.to_sym => 'old',
        'values.offer_family_slug'.to_sym => 'dev_housing_offers'
      )
      expect(config.reload.values.keys).to include('transition_handler').and(include('offer_family_slug'))
      expect(config.reload.values['transition_handler']).to eq('old')
      expect(config.reload.values['offer_family_slug']).to eq('dev_housing_offers')
      create(
        :housing_offer,
        lessor: linked_lessor,
        lessor_status: 'PLACE',
        typology: 'T2',
        lessor_status_updated_at: DateTime.current # TODO: à supprimer une fois les statuts mis à jour
      )
    end
    let!(:basic_housing_offer3) do
      create(
          :housing_offer,
          lessor: linked_lessor,
          typology: 'T2',
          is_mad: true
      )
    end
    let!(:linked_lessor_objective) do
      config = Setting.find_or_create_by(key: 'config', env: Rails.env)
      config.set(
        'values.transition_handler'.to_sym => 'old',
        'values.offer_family_slug'.to_sym => 'dev_housing_offers'
      )
      expect(config.reload.values.keys).to include('transition_handler').and(include('offer_family_slug'))
      expect(config.reload.values['transition_handler']).to eq('old')
      expect(config.reload.values['offer_family_slug']).to eq('dev_housing_offers')
      create(
        :lessor_objective,
        activation_date: DateTime.current,
        lessor: linked_lessor,
        target_by_typologies: {:T2 =>100},
      )
    end

    it 'count when offer placed' do
      expect(linked_lessor_objective.target_by_typologies).to(eq({:T2=>100}))
      expect(linked_lessor_objective.present_typologies).to(eq([:T2]))
      expect(linked_lessor_objective.realized_by_typologies).to(eq({:T2=>2}))
    end

    it 'edit offer with nud trigger sopra flux' do
      after_save_res = basic_housing_offer3.after_save_actions
      #expect(after_save_res[:exectuted]).to eq(true)
    end
  end

  describe 'with large quantity' do
    nb = 150
    nb.times do |i|
      let!("offer#{i}") { create(:housing_offer) }
    end

    describe 'als_id is set' do
      let(:all_offers) do
        Hermes::HousingOffer.only(:als_id)
                            .where(als_id: { :$ne => nil })
      end

      it "has #{nb} entities" do
        expect(all_offers.count).to be nb
      end

      let(:als_ids) { all_offers.pluck(:als_id).uniq.compact }

      it 'produces valid lenght' do
        expect(als_ids.map(&:length)).to all(be(18))
      end

      it 'has no duplicates' do
        expect(
          als_ids.map { |als_id| all_offers.where(als_id: als_id).count }
        ).to all(be(1))
      end
    end
  end

  describe 'on creation PUBLIE' do
    let!(:offer_1) do
      create(
        :housing_offer,
        typology: 'T2',
        rent_with_charges: 434.32 + 53.12 + 22.43,
        rent_amount: 434.32,
        rental_charges: 53.12,
        parking_charges: 22.43,
        surface: 34,
        offer_status: 'PUBLIE',
        threshold_type: 'PLA LM',
        department: '34',
        threshold_subtype: 'Autres régions'
      )
    end

    it 'eligible_query ' do
      expect(offer_1.reload.eligibility_query).not_to be_nil
    end
  end

  describe 'Expiration settings on save' do
    let!(:lessor) { create :lessor }
    let(:published_offer) { create :published_offer, lessor_id: lessor.id }
    let(:long_publishing) { create :published_offer, lessor_id: lessor.id, expiry_days: 25 }

    shared_examples 'publication_end_date priority' do
      before do
        [published_offer, long_publishing].each do |offer|
          offer.update(publication_end_date: Time.new(2020, 3, 15, 0, 0, 0) + 20.days)
        end
      end
      it 'always sets the date from the field' do
        [published_offer, long_publishing].each do |offer|
          expect(offer.publication_end_date).to eq (Time.new(2020, 3, 15, 0, 0, 0) + 20.days)
        end
      end
    end

    context 'when no expiration setting for department' do
      it_behaves_like 'publication_end_date priority'

      it 'sets the minimal_publication_days' do
        expect(published_offer.minimal_publication_days).to eq 3
      end

      it 'computes the publication_end_date with default expiration' do
        expect(published_offer.publication_end_date.strftime('%d/%m/%Y')).to eq (Time.new(2020, 3, 15, 0, 0, 0) + 7.days).strftime('%d/%m/%Y')
      end

      it 'computes the publication_end_date with defined expiration' do
        expect(long_publishing.publication_end_date.strftime('%d/%m/%Y')).to eq (Time.new(2020, 3, 15, 0, 0, 0) + 25.days).strftime('%d/%m/%Y')
      end
    end

    context 'when expiration setting for department' do
      let!(:setting) { create :expiration_setting, expiry_days: 11 }

      it_behaves_like 'publication_end_date priority'

      it 'sets the minimal_publication_days' do
        expect(published_offer.minimal_publication_days).to eq 6
      end

      it 'sets the max_applicants' do
        expect(published_offer.max_applicants).to eq 10
      end

      it 'computes the publication_end_date with the setting expiration' do
        [published_offer, long_publishing].each do |offer|
          expect(offer.publication_end_date).to eq (Time.new(2020, 3, 15, 0, 0, 0) + 11.days)
        end
      end
    end

    context 'when expiration setting for department and lessor' do
      let!(:setting) { create :expiration_setting }
      let!(:setting_for_lessor) do
        create :expiration_setting, expiry_days: 8, minimal_publication_days: 4, max_applicants: 5, lessor_id: lessor.id
      end

      it_behaves_like 'publication_end_date priority'

      it 'sets the minimal_publication_days' do
        expect(published_offer.minimal_publication_days).to eq 4
      end

      it 'sets the max_applicants' do
        expect(published_offer.max_applicants).to eq 5
      end

      it 'computes the publication_end_date with the setting expiration' do
        [published_offer, long_publishing].each do |offer|
          expect(offer.publication_end_date).to eq(Time.new(2020, 3, 15, 0, 0, 0) + 8.days)
        end
      end
    end
  end

  describe 'Geocoding' do
    let!(:housing_offer) do
      create(
        :housing_offer,
        address: "vieux chemin d'arles",
        postal_code: '13210'
      )
    end
    it 'should call geocoder if address change' do
      housing_offer.address = 'routes des palus'
      housing_offer.save!
      housing_offer.reload

      job = ActiveJob::Base.queue_adapter.enqueued_jobs.last
      expect(job[:job]).to eq(GeocodeJob)
      expect(job[:queue]).to eq('geocoder')
      expect(job[:args]).to eq(
        [
          housing_offer.class.name,
          housing_offer.id.to_s,
          true
        ]
      )
    end
  end

  describe 'threshold type and subtype' do
    let!(:threshold_plus) do
      create(
        :threshold,
        type: "PLUS",
        codes: ["PLU"],
        thresholds_map: {
          "Paris et communes limitrophes"=>{"thresholds_list"=>[23721, 35452, 46473, 55486, 66017, 74286], "another_person"=>8278},
          "IDF hors Paris et Communes limitrophes"=>{"thresholds_list"=>[23721, 35452, 42616, 51046, 60429, 68001], "another_person"=>7577},
          "Autres régions"=>{"thresholds_list"=>[20623, 27540, 33119, 39982, 47035, 53008], "another_person"=>5912}
        }
      )
    end

    let!(:threshold_plai) do
      create(
        :threshold,
        type: "PLA I",
        codes: ["PLN"],
        thresholds_map: {
          "Paris et communes limitrophes"=>{"thresholds_list"=>[13050, 21272, 27883, 30521, 36307, 40859], "another_person"=>4552},
          "IDF hors Paris et Communes limitrophes"=>{"thresholds_list"=>[13050, 21272, 25569, 28075, 33238, 37401], "another_person"=>4166},
          "Autres régions"=>{"thresholds_list"=>[11342, 16525, 19872, 22111, 25870, 29155], "another_person"=>3252}
        }
      )
    end

    let!(:offer_1) do
      create(
        :housing_offer,
        typology: 'T2',
        rent_with_charges: 434.32 + 53.12 + 22.43,
        rent_amount: 434.32,
        rental_charges: 53.12,
        parking_charges: 22.43,
        surface: 34,
        offer_status: 'PUBLIE',
        threshold_type: 'PLU',
        department: '34',
        threshold_subtype: 'Z3'
      )
    end

    it 'on creation' do
      expect(offer_1.reload.eligibility_query).not_to be_nil;
      expect(offer_1.reload.ceil_category_1).to eq(threshold_plus.thresholds_map["Autres régions"]["thresholds_list"][0]);
    end

    it 'threshold_type update' do
      offer_1.threshold_type = 'PLN'
      offer_1.save
      expect(offer_1.reload.eligibility_query).not_to be_nil;
      expect(offer_1.reload.ceil_category_1).to eq(threshold_plai.thresholds_map["Autres régions"]["thresholds_list"][0]);
    end

    it 'threshold_subtype update' do
      offer_1.threshold_subtype = 'Z1'
      offer_1.save
      expect(offer_1.reload.eligibility_query).not_to be_nil;
      expect(offer_1.reload.ceil_category_1).to eq(threshold_plus.thresholds_map["Paris et communes limitrophes"]["thresholds_list"][0]);
    end

    it 'threshold_type & threshold_subtype without mapping' do
      offer_1.threshold_type = 'PLA I'
      offer_1.threshold_subtype = 'IDF hors Paris et Communes limitrophes'
      offer_1.save
      expect(offer_1.reload.eligibility_query).not_to be_nil;
      expect(offer_1.reload.ceil_category_1).to eq(threshold_plai.thresholds_map["IDF hors Paris et Communes limitrophes"]["thresholds_list"][0]);
    end
  end

  describe '#handle_unpublished_status' do
    let(:published_offer) { create :published_offer, :with_applications }
    let(:request) { create :active_request }
    let(:request2) { create :active_request }
    let(:request3) { create :active_request }
    let(:new_application) { create :housing_application, housing_request: request, housing_offer: published_offer, status: "EN ATTENTE" }

    context 'on save' do
      it 'enqueues unpublish job at default publication_end_date' do
        published_offer.save
        expect(jobs = enqueued_jobs.select {|job| job[:job] == UpdateScoreFromApplicantJob}.length ).to eq 1
      end
      
      it 'unpublishes the offer when publication_end_date is reached' do
        Timecop.freeze(Time.local(2020, 3, 22, 0, 0))
        expect {
          drain_all_jobs
        }.to change { published_offer.reload.offer_status }.from('PUBLIE').to('COMMERCIALISATION NON PUBLIEE')
        .and change { published_offer.published }.from(true).to false
        Timecop.return
      end
    end
    
    context 'when max applicants is reached' do
      # minimal expiration date = 15/03/2020 + 3 days (default)
      context 'before minimal publication days' do
        before { Timecop.freeze(Time.local(2020, 3, 17, 0, 0)) }
        after  { Timecop.return }
        
        it 'sets the publication end date to the minimal expiration date' do
          expect {
            new_application2 = Hermes::HousingApplication.create(housing_request: request2, housing_offer: published_offer, status: "EN ATTENTE")
            new_application3 = Hermes::HousingApplication.create(housing_request: request3, housing_offer: published_offer, status: "EN ATTENTE")
          }.to change { published_offer.publication_end_date.strftime("%d/%m/%Y") }.from('22/03/2020').to('18/03/2020')
        end
        
        it 'enqueues unpublish job at publication_end_date' do
          new_application2 = Hermes::HousingApplication.create(housing_request: request2, housing_offer: published_offer, status: "EN ATTENTE")
          new_application3 = Hermes::HousingApplication.create(housing_request: request3, housing_offer: published_offer, status: "EN ATTENTE")
          expect(jobs = enqueued_jobs.select {|job| job[:job] == UpdateScoreFromApplicantJob}.length ).to eq 3
        end
      end
      
      context 'after minimal publication days' do
        before { Timecop.freeze(Time.local(2020, 3, 21, 0, 0)) }
        after  { Timecop.return }

        it 'unpublishes the offer' do
          expect {
            new_application2 = Hermes::HousingApplication.create(housing_request: request2, housing_offer: published_offer, status: "EN ATTENTE")
            new_application3 = Hermes::HousingApplication.create(housing_request: request3, housing_offer: published_offer, status: "EN ATTENTE")
          }.to change { published_offer.publication_end_date }.from('22/03/2020'.to_time).to(Time.now)
            .and change { published_offer.offer_status }.from('PUBLIE').to('COMMERCIALISATION NON PUBLIEE')
        end
      end
    end

    describe 'Hooks' do
      context 'Effort rate auto compute'  do
        it 'should have default value' do
          expect(Hermes::HousingOffer.create().effort_rate).not_to be_nil
        end

        it 'should have default value dependant of typology' do
          expect(Hermes::HousingOffer.create(typology: 't1').effort_rate).to eq(HERMES_CONFIG["EFFORT_RATES"]['T1'])
          expect(Hermes::HousingOffer.create(typology: 't6').effort_rate).to eq(HERMES_CONFIG["EFFORT_RATES"]['T6'])
        end

        it 'should have default value with invalid typology' do
          expect(Hermes::HousingOffer.create(typology: 'azerty').effort_rate).not_to be_nil
          expect(Hermes::HousingOffer.create(typology: 'azerty').effort_rate).to eq(Hermes::HousingOffer.create().min_effort_rate)
        end

        it 'use custom effort_rate if passed' do
          expect(Hermes::HousingOffer.create(typology: 't1', effort_rate: 12.1).effort_rate).to eq(12.1)
        end
        
        it 'use custom or auto when is passed' do
          offer = FactoryBot.create(:housing_offer, typology: 't3', effort_rate: nil)
          expect(offer.reload.effort_rate).to eq(HERMES_CONFIG["EFFORT_RATES"]['T3'])
          
          offer.update(effort_rate: 12.1)
          expect(offer.reload.effort_rate).to eq(12.1)
          
          offer.update(effort_rate: nil, typology: 't1')
          expect(offer.reload.effort_rate).to eq(HERMES_CONFIG["EFFORT_RATES"]['T1'])
        end
      end

      context 'Charges auto compute' do
        let!(:valid_offer) { create(:housing_offer, rent_with_charges: 1250, rental_charges: 50, rent_amount: nil) }
        let!(:invalid_offer) { create(:housing_offer, rent_with_charges: nil, rental_charges: nil, rent_amount: nil) }

        it 'should auto fill rent_amount if blank' do
          expect(valid_offer.rent_amount).not_to be_nil
          expect(valid_offer.rent_amount).to eq(1250.0)
        end

        it 'should not auto fill rent_amount if cannot' do
          expect(invalid_offer.rent_amount).to be_nil
        end
      end
    end
  end

  describe 'lessor rights' do
    let(:lessor) { create :lessor }
    let(:user) { create :user }
    let(:group_user) { create :user, group_ids:[lessor.group_id] }
    let(:supervisor_user) { create :user, group_ids:[lessor.supervisor_group_id] }
    let(:manager_user) { create :user, group_ids:[lessor.manager_group_id] }
    let(:offer) { create :housing_offer }

    it 'not add right without lessor' do
      expect(Hermes::HousingOffer.find(offer.id).readable_by?(user)).to eq(false)
      expect(Hermes::HousingOffer.find(offer.id).readable_by?(group_user)).to eq(false)
      expect(Hermes::HousingOffer.find(offer.id).readable_by?(supervisor_user)).to eq(false)
      expect(Hermes::HousingOffer.find(offer.id).readable_by?(manager_user)).to eq(false)
    end

    it 'add right with lessor' do
      offer.lessor = lessor
      offer.save
      expect(Hermes::HousingOffer.find(offer.id).readable_by?(user)).to eq(false)
      expect(Hermes::HousingOffer.find(offer.id).readable_by?(group_user)).to eq(true)
      expect(Hermes::HousingOffer.find(offer.id).writable_by?(supervisor_user)).to eq(true)
      expect(Hermes::HousingOffer.find(offer.id).writable_by?(manager_user)).to eq(true)
    end
  end

  describe 'test uniqueness' do 
    let!(:offer1) { create(:housing_offer)}
    let!(:offer2) { create(:housing_offer)}
    it 'work' do 
      start = Time.now
      10.times do
        UpdateRankJob.perform_later(offer1.id.to_s)
        UpdateRankJob.perform_later(offer2.id.to_s)
      end
      finish = Time.now

      diff = finish - start
      expect(diff < 1).to eq(true)
      expect(enqueued_jobs.select {|job| job[:job] == UpdateRankJob}.length).to eq(2)
    end 
  end 


  describe 'housing_channel_keys' do 
    let!(:offer1) { create(:housing_offer)}
    let!(:offer2) { create(:housing_offer, housing_channel_keys: ['dalo'])}
    let!(:offer3) { create(:housing_offer, housing_channel_keys: ['dc','dalo'])}
    let!(:offer3) { create(:housing_offer, housing_channel_keys: ['dalo', 'lol'])}

    it 'create with and without housing_channel_keys' do 
      expect(offer1.housing_channel_keys).to eq(['dc'])
      expect(offer2.housing_channel_keys).to eq(['dalo'])
      expect(offer3.housing_channel_keys).to eq(['dalo', 'lol'])
    end 

    it 'when delete housing_channel_keys' do 
      offer2.update(housing_channel_keys: [])
      expect(offer2.reload.housing_channel_keys).to eq(['dc'])
    end 

    it 'when add housing_channel_keys' do 
      offer1.update(housing_channel_keys: ['dalo'])
      expect(offer1.reload.housing_channel_keys).to eq(['dalo'])
    end 

  end 


  
end
