# frozen_string_literal: true

require 'rails_helper'
require [Hermes::Engine.root, %w[lib extras string]].flatten.join('/')

RSpec.describe Hermes::Score, type: :model do
  let!(:plus) do
    create(
      :threshold,
      type: 'PLUS',
      thresholds_map: {
        'Paris et communes limitrophes' => {
          'thresholds_list' => [23_721, 35_452, 46_473, 55_486, 66_017, 74_286],
          'another_person' => 8278
        }, 'IDF hors Paris et Communes limitrophes' => {
          'thresholds_list' => [23_721, 35_452, 42_616, 51_046, 60_429, 68_001],
          'another_person' => 7577
        }, 'Autres régions' => {
          'thresholds_list' => [20_623, 27_540, 33_119, 39_982, 47_035, 53_008],
          'another_person' => 5912
        }
      }
    )
  end
  let!(:plai) do
    create(
      :threshold,
      type: 'PLA I',
      thresholds_map: {
        'Paris et communes limitrophes' => {
          'thresholds_list' => [13_050, 21_272, 27_883, 30_521, 36_307, 40_859],
          'another_person' => 4552
        }, 'IDF hors Paris et Communes limitrophes' => {
          'thresholds_list' => [13_050, 21_272, 25_569, 28_075, 33_238, 37_401],
          'another_person' => 4166
        }, 'Autres régions' => {
          'thresholds_list' => [11_342, 16_525, 19_872, 22_111, 25_870, 29_155],
          'another_person' => 3252
        }
      }
    )
  end
  context 'with basic setup' do
    let!(:applicant_1) do
      create(
        :applicant,
        monthly_incomes: {
          "Salaire": 1000
        },
        postal_code: '83100',
        birth_date: nil
      )
    end
    let!(:housing_request_1) do
      create(
        :housing_request,
        is_active: true,
        main_applicant_id: applicant_1.id,
        rejected_in_cal_nb: 1,
        rejected_after_cal_nb: 0,
        priority_points_indexes: [30, 145]
      )
    end

    let!(:entity_1) do
      create(
        :entity
      )
    end

    let!(:applicant_2) do
      create(
        :applicant,
        birth_date: Date.parse((Date.current - 10.years).to_s),
        entity: entity_1,
        monthly_incomes: {
          "Salaire": 2000
        },
        postal_code: '83100'
      )
    end
    let!(:housing_request_2) do
      create(
        :housing_request,
        sne_created_at: Time.now - 13.months,
        is_active: true,
        main_applicant_id: applicant_2.id,
        rejected_in_cal_nb: 0,
        rejected_after_cal_nb: 1,
        motivation1: '19',
        entity_validated: true,
        priority_points_indexes: [30, 145]
      )
    end

    let!(:housing_offer_1) do
      create(
        :housing_offer,
        rent_with_charges: 500,
        department: '83',
        postal_code: '83100'
      )
    end

    let!(:entity_validation_2) do
      create(
        :entity_validation,
        applicant_id: applicant_2.id,
        active: true,
        validation: true,
      )
    end

    it 'has a valid factory' do
      expect(
        FactoryBot.build(
          :score,
          applicant: applicant_1,
          housing_offer: housing_offer_1
        )
      ).to be_valid
    end

    describe 'Base relationship and creation' do
      it 'should has rightly set relationships' do
        expect(applicant_1.housing_requests.length).to be > 0
        expect(applicant_2.housing_requests.length).to be > 0
      end

      it 'should has set applicants to have at least one active request' do
        expect(applicant_1.housing_requests.actives.first.is_active).to be true
        expect(applicant_2.housing_requests.actives.first.is_active).to be true
        expect(applicant_1.first_active_request).to_not be_nil
        expect(applicant_2.first_active_request).to_not be_nil
      end
    end

    let!(:score_1) do
      create(
        :score,
        is_active: true,
        applicant_id: applicant_1.id,
        housing_offer_id: housing_offer_1.id
      )
    end

    let!(:score_2) do
      create(
        :score,
        is_active: true,
        applicant_id: applicant_2.id,
        housing_offer_id: housing_offer_1.id,
        epci: 24
      )
    end

    describe 'Test scoring process' do
      before :each do
        score_1.process
        score_2.process
      end

      it 'should update data with right types' do
        expect([score_1.profile_score, score_2.profile_score]).to(
          all(be_a(Integer))
        )
        expect([score_1.profile_epci_score, score_2.profile_epci_score]).to(
          all(be_a(Integer))
        )
      end

      it 'should eventualy has set the right profile_score' do
        expect(score_1.applicant_is_under_30).to be false
        expect(score_1.already_rejected_in_cal).to be true
        expect(score_1.already_declined_cal).to be false
        expect(score_1.calculate_under_30_year_points).to eq 0
        expect(score_1.entity_points).to eq 0
        expect(score_1.calculate_history_points).to eq 40
        expect(score_1.calculate_effort_rate_points).to eq 0
        expect(score_1.calculate_motivation_points).to be 0
        expect(score_1.calculate_seniority_points).to be 0
        expect(score_1.profile_score).to be 40
        expect(score_1.profile_score.to_i + score_1.epci.to_i).to be(
          score_1.profile_epci_score
        )
        
        expect(score_2.applicant_is_under_30).to be true
        expect(score_2.already_rejected_in_cal).to be false
        expect(score_2.already_declined_cal).to be true
        expect(score_2.already_declined_cal).to be true
        expect(score_2.calculate_under_30_year_points).to eq 70
        expect(score_2.entity_points).to be  240 + 175
        expect(score_2.calculate_history_points).to eq(-40)
        expect(score_2.calculate_motivation_points).to be 70
        expect(score_2.calculate_seniority_points).to be 13
        expect(score_2.profile_score).to be 70 + 240 + 175 - 40 + 0 + 70 + 13
        expect(score_2.profile_score.to_i + score_2.epci.to_i).to be(
          score_2.profile_epci_score
        )
      end

      it 'returns eligible position by location' do
        expect(score_1.applicant_postal_code).to eq applicant_1.postal_code
        expect(score_1.applicant_department).to eq applicant_1.department
        expect(score_2.located_eligible_position).to eq(
          {
            position_by_department: 1,
            position_by_postal_code: 1,
            total_by_department: 2,
            total_by_postal_code: 2
          }
        )
      end

      it 'sets detailed score' do
        score = [score_1, score_2].sample

        expect(score.detailed_score).to_not be nil
        expect(score.detailed_score.under_30_year_points).to eq score.calculate_under_30_year_points
        expect(score.detailed_score.history_points).to eq score.calculate_history_points
        expect(score.detailed_score.entity_points).to eq score.entity_points
        expect(score.detailed_score.effort_rate_points).to eq score.calculate_effort_rate_points
      end
    end
  end

  context 'with precise setup' do
    # ENTITIES
    let!(:entity1) { create(:entity, siret: '12345678900000') }
    let!(:entity2) { create(:entity, siret: '23984799300000') }
    let!(:entity3) { create(:entity, siret: '12345567780000') }
    # FAMILY 1
    let!(:coapp1) do
      create(
        :applicant,
        birth_date: '1985-12-14',
        first_name: 'Jacqueline',
        last_name: 'Durand',
        family_situation: 'Marié (e)',
        monthly_incomes: {
          'Salaire ou revenu d\'activité': 1250
        },
        entity: entity2
      )
    end
    let!(:account1) { create(:user, first_name: 'Jean', last_name: 'Dupont', email: 'jean.dupont@test.test') }
    let!(:app1) do
      create(
        :applicant,
        user_id: account1.id.to_s,
        nud: '1223455',
        sne_profile_creation_date: '2016-04-03',
        family_situation: 'Marié (e)',
        birth_date: '1995-03-12',
        year_minus_one: '2018',
        amount_year_minus_one: 29400,
        year_minus_two: '2017',
        amount_year_minus_two: 29400,
        monthly_incomes: {
          'Salaire ou revenu d\'activité': 1200
        },
        entity: entity1
      )
    end
    let!(:request1) do
      create(
        :housing_request,
        is_active: true,
        main_applicant: app1,
        seeked_departments: %w[
          75
        ],
        seeked_district_postal_codes: %w[
          92002
          92014
          92071
        ],
        coapplicants: [
          { rel_type: 'Marié (e)', id: coapp1.id.to_s }
        ],
        dependents: [
          {
            rel_type: 'enfant à charge',
            birth_date: '2017-03-23',
            first_name: 'Paul',
            last_name: 'Dupont'
          },
          {
            rel_type: 'enfant à charge',
            birth_date: '2019-08-04',
            first_name: 'Cerise',
            last_name: 'Dupont'
          }
        ]
      )
    end
    # FAMILY 2
    let!(:coapp2) do
      create(
        :applicant,
        birth_date: '1985-12-14',
        first_name: 'Jeanne',
        last_name: 'Alphonso',
        family_situation: 'Pacsé (e)',
        monthly_incomes: {
          'Salaire ou revenu d\'activité': 1050
        },
        entity: entity2
      )
    end
    let!(:account2) { create(:user, first_name: 'Eric', last_name: 'Alphonso', email: 'eric.alphonso@test.test') }
    let!(:app2) do
      create(
        :applicant,
        user_id: account2.id.to_s,
        nud: '445559980',
        sne_profile_creation_date: '2017-06-13',
        family_situation: 'Pacsé (e)',
        birth_date: '1995-03-12',
        year_minus_one: '2018',
        amount_year_minus_one: 28200,
        year_minus_two: '2017',
        amount_year_minus_two: 28200,
        monthly_incomes: {
          'Salaire ou revenu d\'activité': 1300
        },
        entity: entity3
      )
    end
    let!(:request2) do
      create(
        :housing_request,
        is_active: true,
        main_applicant: app2,
        seeked_departments: %w[
          75
        ],
        seeked_district_postal_codes: %w[
          92002
          92014
          92071
        ],
        coapplicants: [
          { rel_type: 'Pacsé (e)', id: coapp2.id.to_s }
        ],
        dependents: [
          {
            rel_type: 'enfant à charge',
            birth_date: '2017-03-23',
            first_name: 'Grégoire',
            last_name: 'Alphonso'
          }
        ]
      )
    end
    # FAMILY 3
    let!(:coapp3) do
      create(
        :applicant,
        birth_date: '1985-12-14',
        first_name: 'Paulette',
        last_name: 'Rimeri',
        family_situation: 'Pacsé (e)',
        monthly_incomes: {
          'Salaire ou revenu d\'activité': 750
        },
        entity: entity2
      )
    end
    let!(:account3) { create(:user, first_name: 'François', last_name: 'D\'arville', email: 'francois.darville@test.test') }
    let!(:app3) do
      create(
        :applicant,
        user_id: account3.id.to_s,
        nud: '678993039',
        sne_profile_creation_date: '2017-08-23',
        family_situation: 'Pacsé (e)',
        birth_date: '1995-03-12',
        year_minus_one: '2018',
        amount_year_minus_one: 33900,
        year_minus_two: '2017',
        amount_year_minus_two: 33900,
        monthly_incomes: {
          'Salaire ou revenu d\'activité': 725
        },
        entity: entity3
      )
    end
    let!(:request3) do
      create(
        :housing_request,
        is_active: true,
        mutation: true,
        main_applicant: app3,
        seeked_departments: %w[
          75
        ],
        seeked_district_postal_codes: %w[
          92002
          92014
          92071
        ],
        coapplicants: [
          { rel_type: 'Pacsé (e)', id: coapp3.id.to_s }
        ],
        dependents: [
          {
            rel_type: 'enfant à charge',
            birth_date: '2017-03-23',
            first_name: 'George',
            last_name: 'Rimeri',
            monthly_incomes: {
              'Salaire ou revenu d\'activité': 1425
            }
          }
        ]
      )
    end

    let!(:nud1) { create(:nud,housing_request: request1, unique_number: '1')}
    let!(:nud2) { create(:nud,housing_request: request2, unique_number: '2')}
    let!(:nud3) { create(:nud,housing_request: request3, unique_number: '3')}

    # OFFERS
    let!(:offer1) do
      create(
        :housing_offer,
        typology: 'T3',
        surface: 57,
        postal_code: '92002',
        department: '92',
        threshold_type: 'PLUS',
        threshold_subtype: 'IDF hors Paris et Communes limitrophes',
        offer_status: 'LOGEMENT PUBLIE',
        min_effort_rate: 0.333,
        rent_with_charges: 640+ 50,
        rent_amount: 640,
        rental_charges: 50
      )
    end
    let!(:offer2) do
      create(
        :housing_offer,
        typology: 'T3',
        surface: 53,
        postal_code: '92071',
        department: '92',
        threshold_type: 'PLUS',
        threshold_subtype: 'IDF hors Paris et Communes limitrophes',
        offer_status: 'LOGEMENT PUBLIE',
        min_effort_rate: 0.333,
        rent_with_charges: 750 + 60,
        rent_amount: 750,
        rental_charges: 60
      )
    end
    let!(:offer3) do
      create(
        :housing_offer,
        typology: 'T3',
        surface: 70,
        postal_code: '92002',
        department: '92',
        threshold_type: 'PLA I',
        threshold_subtype: 'IDF hors Paris et Communes limitrophes',
        offer_status: 'LOGEMENT PUBLIE',
        min_effort_rate: 0.333,
        rent_with_charges: 560 + 90,
        rent_amount: 560,
        rental_charges: 90
      )
    end
    let!(:offer4) do
      create(
        :housing_offer,
        typology: 'T2',
        surface: 80,
        postal_code: '92014',
        department: '92',
        threshold_type: 'PLUS',
        threshold_subtype: 'IDF hors Paris et Communes limitrophes',
        offer_status: 'LOGEMENT PUBLIE',
        min_effort_rate: 0.333,
        rent_with_charges: 900 + 110,
        rent_amount: '900',
        rental_charges: '110'
      )
    end
    let!(:offer5) do
      create(
        :housing_offer,
        typology: 'T3',
        surface: 80,
        postal_code: '92012',
        department: '92',
        threshold_type: 'PLUS',
        threshold_subtype: 'Paris et communes limitrophes',
        offer_status: 'LOGEMENT PUBLIE',
        min_effort_rate: 0.333,
        rent_with_charges: 1000 + 110,
        rent_amount: 1000,
        rental_charges: 110
      )
    end
    let!(:offer6) do
      create(
        :housing_offer,
        typology: 'T3',
        surface: 85,
        postal_code: '92012',
        department: '92',
        threshold_type: 'PLA I',
        threshold_subtype: 'IDF hors Paris et Communes limitrophes',
        offer_status: 'LOGEMENT PUBLIE',
        min_effort_rate: 0.333,
        rent_with_charges: 700 + 110,
        rent_amount: 700,
        rental_charges: 110
      )
    end
    let!(:offer7) do
      create(
        :housing_offer,
        typology: 'T3',
        surface: 90,
        postal_code: '92071',
        department: '92',
        threshold_type: 'PLA I',
        threshold_subtype: 'IDF hors Paris et Communes limitrophes',
        offer_status: 'LOGEMENT PUBLIE',
        min_effort_rate: 0.333,
        rent_with_charges: 500 + 110,
        rent_amount: 500,
        rental_charges: 110
      )
    end
    let!(:offer8) do
      create(
        :housing_offer,
        typology: 'T4',
        surface: 95,
        postal_code: '92071',
        department: '92',
        threshold_type: 'PLUS',
        threshold_subtype: 'IDF hors Paris et Communes limitrophes',
        offer_status: 'LOGEMENT PUBLIE',
        min_effort_rate: 0.333,
        rent_with_charges: 1450 + 110,
        rent_amount: 1450,
        rental_charges: 110
      )
    end
    let!(:offer9) do
      create(
        :housing_offer,
        typology: 'T3',
        surface: 45,
        department: '75',
        threshold_type: 'PLA I',
        threshold_subtype: 'Paris et communes limitrophes',
        offer_status: 'LOGEMENT PUBLIE',
        min_effort_rate: 0.333,
        rent_with_charges: 760 + 50,
        rent_amount: 760,
        rental_charges: 50
      )
    end
    let!(:offer10) do
      create(
        :housing_offer,
        typology: 'T3',
        surface: 45,
        department: '75',
        threshold_type: 'PLUS',
        threshold_subtype: 'Paris et communes limitrophes',
        offer_status: 'LOGEMENT PUBLIE',
        min_effort_rate: 0.333,
        rent_with_charges: 760 + 50,
        rent_amount: 760,
        rental_charges: 50
      )
    end
    let!(:offer11) do
      create(
        :housing_offer,
        typology: 'T2',
        surface: 45,
        department: '75',
        threshold_type: 'PLUS',
        threshold_subtype: 'Paris et communes limitrophes',
        offer_status: 'LOGEMENT PUBLIE',
        min_effort_rate: 0.333,
        rent_with_charges: 760 + 50,
        rent_amount: 760,
        rental_charges: 50
      )
    end

    let!(:nud1) { create(:nud,housing_request: request1, unique_number: '1')}
    let!(:nud2) { create(:nud,housing_request: request2, unique_number: '2')}
    let!(:nud3) { create(:nud,housing_request: request3, unique_number: '3')}

    before do
      [request1, request2, request3].each do |hr|
        hr.validate_all_required_document
      end
      drain_all_jobs
    end

    describe 'check eligibility and scores for all families' do
      describe 'eligibility and score for family1' do
        it 'should has valid eligiblity' do
          r_app1 = app1.reload
          where = r_app1.eligibility_query.json_to_h
          eligible_offers = Hermes::HousingOffer.where(where)
          eligible_ids = eligible_offers.pluck(:id).map(&:to_s)
          expect(request1.reload.category_type).to be 4
          expect(request1.reload.reference_tax_revenues).to(
            be > plai.thresholds_map['IDF hors Paris et Communes limitrophes']['thresholds_list'][3]
          )
          expect(eligible_offers.count).to be 4
          [offer1, offer2, offer9, offer10].each do |offer|
            expect(eligible_ids).to include(offer.id.to_s)
          end
        end
      end

      describe 'eligibility and score for family2' do
        let(:r_app2) { app2.reload }

        it 'should has valid eligiblity' do
          where = r_app2.eligibility_query.json_to_h
          eligible_offers = Hermes::HousingOffer.where(where.merge(offer_status: 'LOGEMENT PUBLIE'))
          eligible_ids = eligible_offers.pluck(:id).map(&:to_s)
          expect(request2.category_type).to be 3
          expect(eligible_offers.count).to be 1
          [offer1].each do |offer|
            expect(eligible_ids).to include(offer.id.to_s)
          end
        end

        it 'has generated valid scores' do
          r2_scores = Hermes::Score.where(applicant: r_app2)
          expect(r2_scores.count).to be 1
          expect(r2_scores.pluck(:mutation)).not_to include(false)
        end

        xit 'notify applicant' do
          expect(r_app2.notifications.map(&:key)).to include(NOTIFICATION_KEYS[:housing_offer][:new_eligible_offer])
        end
      end

      describe 'eligibility and score for family3' do
        let(:r_app3) { app3.reload }

        it 'should has valid eligiblity' do
          where = r_app3.eligibility_query.json_to_h
          eligible_offers = Hermes::HousingOffer.where(where)
          eligible_ids = eligible_offers.pluck(:id).map(&:to_s)
          expect(request3.category_type).to be 3
          expect(eligible_offers.count).to be 4
          [offer1, offer2, offer10, offer11].each do |offer|
            expect(eligible_ids).to include(offer.id.to_s)
          end
        end

        it 'has generated valid scores' do
          r3_scores = Hermes::Score.where(applicant: r_app3)
          expect(r3_scores.count).to be 4
          expect(r3_scores.pluck(:mutation)).not_to include(false)
        end
      end
    end
  end

  context 'for eligibility_count' do
    let!(:entity1) { create(:entity, siret: '12345678900000') }
    let!(:entity2) { create(:entity, siret: '23984799300000') }
    let!(:coapp1) do
      create(
        :applicant,
        birth_date: '1985-12-14',
        first_name: 'Jacqueline',
        last_name: 'Durand',
        family_situation: 'Marié (e)',
        monthly_incomes: {
          'Salaire ou revenu d\'activité': 1250
        },
        entity: entity2
      )
    end
    let!(:account1) { create(:user, first_name: 'Jean', last_name: 'Dupont', email: 'jean.dupont2@test.test') }
    let!(:app1) do
      create(
        :applicant,
        user_id: account1.id.to_s,
        nud: '1223455',
        sne_profile_creation_date: '2016-04-03',
        family_situation: 'Marié (e)',
        birth_date: '1995-03-12',
        year_minus_one: '2018',
        amount_year_minus_one: 29_400,
        year_minus_two: '2017',
        amount_year_minus_two: 29_400,
        monthly_incomes: {
          'Salaire ou revenu d\'activité': 1200
        },
        entity: entity1
      )
    end
    let!(:request1) do
      create(
        :housing_request,
        is_active: true,
        entity_validated: true,
        main_applicant: app1,
        mutation: true,
        seeked_departments: %w[
          75
        ],
        seeked_district_postal_codes: %w[
          92002
          92014
          92071
        ],
        coapplicants: [
          { rel_type: 'Marié (e)', id: coapp1.id.to_s }
        ],
        dependents: [
          {
            rel_type: 'enfant à charge',
            birth_date: '2017-03-23',
            first_name: 'Paul',
            last_name: 'Dupont'
          },
          {
            rel_type: 'enfant à charge',
            birth_date: '2019-08-04',
            first_name: 'Cerise',
            last_name: 'Dupont'
          }
        ]
      )
    end

    let!(:account2) { create(:user, first_name: 'Jean', last_name: 'Dupont', email: 'jean.dupont3@test.test') }
    let!(:app2) do
      create(
        :applicant,
        user_id: account1.id.to_s,
        nud: '1223455',
        sne_profile_creation_date: '2016-04-03',
        family_situation: 'Marié (e)',
        birth_date: '1995-03-12',
        year_minus_one: '2018',
        amount_year_minus_one: 29_400,
        year_minus_two: '2017',
        amount_year_minus_two: 29_400,
        monthly_incomes: {
          'Salaire ou revenu d\'activité': 1200
        },
        entity: entity2
      )
    end
    let!(:request2) do
      create(
        :housing_request,
        is_active: true,
        main_applicant: app2,
        seeked_departments: %w[
          75
        ],
        seeked_district_postal_codes: %w[
          92002
          92014
          92071
        ],
        coapplicants: [
          { rel_type: 'Marié (e)', id: coapp1.id.to_s }
        ],
        dependents: [
          {
            rel_type: 'enfant à charge',
            birth_date: '2017-03-23',
            first_name: 'Paul',
            last_name: 'Dupont'
          },
          {
            rel_type: 'enfant à charge',
            birth_date: '2019-08-04',
            first_name: 'Cerise',
            last_name: 'Dupont'
          }
        ]
      )
    end

    let!(:nud1) { create(:nud,housing_request: request1, unique_number: '1')}
    let!(:nud2) { create(:nud,housing_request: request2, unique_number: '2')}

    let!(:offer1) do
      create(
        :housing_offer,
        typology: 'T3',
        surface: 57,
        postal_code: '92002',
        department: '92',
        threshold_type: 'PLUS',
        threshold_subtype: 'IDF hors Paris et Communes limitrophes',
        offer_status: 'LOGEMENT PUBLIE',
        min_effort_rate: 0.333,
        rent_with_charges: 640 + 50,
        rent_amount: 640,
        rental_charges: 50
      )
    end

    before do
      [request1, request2].each do |hr|
        hr.validate_all_required_document
      end
      drain_all_jobs
    end

    describe 'eligibility quantities are valide for offer1' do
      let(:r_offer1) { offer1.reload }

      it 'has valid quantity of eligibility' do
        expect(r_offer1.eligible_requests_nb).to be 2
      end

      it 'has valid eligible details' do
        expect(r_offer1.eligible_requests_nb_detailed['mutation']).to be 1
        expect(r_offer1.eligible_requests_nb_detailed['entities'][app1.entity_id.to_s]).to be 1
        expect(r_offer1.eligible_requests_nb_detailed['entities'][app2.entity_id.to_s]).to_not be_present
      end
    end
  end

  def time_rand(from = Time.parse('1997-01-01'), to = Time.now)
    Time.at(from + rand * (to.to_f - from.to_f))
  end

  def rand_mail
    rand_mail_ext = -> { %w[com net org fr].sample }
    "#{Faker::Name.name_with_middle.downcase.gsub(' ', '.')}@test.#{rand_mail_ext.call}"
  end

  def rand_typology
    "T#{rand(1..12)}"
  end

  def rand_subtype
    [
      'Paris et communes limitrophes',
      'IDF hors Paris et Communes limitrophes',
      'Autres régions'
    ].sample
  end

  context 'with large quantity of requests' do
    let!(:threshold1) do
      create(
        :threshold,
        type: 'PLA LM',
        thresholds_map: {
          'Paris et communes limitrophes': {
            'thresholds_list': [
              13_050,
              21_272,
              27_883,
              30_521,
              36_307,
              40_859
            ],
            'another_person': 4552
          },
          'IDF hors Paris et Communes limitrophes': {
            'thresholds_list': [
              13_050,
              21_272,
              25_569,
              28_075,
              33_238,
              37_401
            ],
            'another_person': 4166
          },
          'Autres régions': {
            'thresholds_list': [
              11_342,
              16_525,
              19_872,
              22_111,
              25_870,
              29_155
            ],
            'another_person': 3252
          }
        }
      )
    end

    let!(:applicants_list) do
      100.times.map do
        user = create(
          :user,
          email: rand_mail
        )
        applicant = create(
          :applicant,
          user_id: user.id,
          amount_year_minus_two: rand(7000..10_000),
          monthly_incomes: {
            'SAL' => rand(760.00..3500.00)
          }
        )
        request = create(
          :housing_request,
          is_active: true,
          main_applicant: applicant,
          seeked_types: ['T2'],
          seeked_departments: ['34']
        )
        nud = create(
          :nud,
          department: '34',
          creation_date: time_rand,
          housing_request: request
        )
        request.validate_all_required_document
        {
          user: user,
          applicant: applicant,
          request: request,
          nud: nud
        }
      end
    end

    let!(:offers_list) do
      1.times.map do
        create(
          :housing_offer,
          typology: 'T2',
          department: '34',
          offer_status: 'LOGEMENT PUBLIE',
          rent_with_charges: rand(100.00..890.00),
          threshold_type: 'PLA LM',
          min_effort_rate: 0.0,
          threshold_subtype: rand_subtype
        )
      end
    end

    describe 'with all scores generated' do
      before :each do
        t = Time.now
        expect(Hermes::HousingRequest.count).to be applicants_list.length
        expect(Hermes::HousingOffer.count).to be offers_list.length
        drain_all_jobs
      end

      it 'should has valid quantity of created scores and should be correctly ordinated' do
        expect(Hermes::Score.count).to be(applicants_list.length * offers_list.length)
        expect(Hermes::Score.only(:sne_date).pluck(:sne_date)).to all(be_a(DateTime).or(be_a(Time)))
        offer_ids = Hermes::Score.only(:housing_offer_id).pluck(:housing_offer_id).uniq.compact
        offer_ids.each do |offer_id|
          offer_scores = Hermes::Score.where(housing_offer_id: offer_id).order(profile_epci_score: -1, sne_date: 1)
          offer_scores.each_cons(2) do |a, b|
            # puts "now : [#{a.profile_epci_score}]#{a.sne_date} " \
            #   "next : [#{b.profile_epci_score}]#{b.sne_date}"
            expect(b.eligible_position).to be >= a.eligible_position
          end
        end
      end
    end
  end

  context 'apl simulation' do

    describe 'apl simulation on a score that has an applicant' do

      let!(:entity_apl) do
        create(
          :entity
        )
      end

      let!(:applicant_apl) do
        create(
          :applicant,
          birth_date: Date.parse((Date.current - 10.years).to_s),
          entity: entity_apl,
          monthly_incomes: {
            "Salaire": 2000
          },
          postal_code: '83100',
          amount_year_minus_two: 0
        )
      end

      let!(:housing_offer_apl) do
        create(
          :housing_offer,
          rent_amount: 500,
          rent_with_charges: 500,
          department: '83',
          postal_code: '83100',
          city: 'marseille'
        )
      end
      
      let!(:score_apl) do
        create(
          :score,
          is_active: true,
          applicant_id: applicant_apl.id,
          housing_offer_id: housing_offer_apl.id
        )
      end

      it 'builds the age ' do
        expect(score_apl.build_age(applicant_apl)).to eq(10)
      end


      def build_month
        date = "#{Date.today.year}-#{Date.today.month + 1}"
        return date
      end 
  
      def build_year
        date = Date.today.year - 2
        return date
      end 

      def build_body 
        applicant = {}
        applicant["#{applicant_apl.first_name}"] = {
                                                                "age": {
                                                                "#{build_month}": 10
                                                                }, 
                                                                "revenu_assimile_salaire": {
                                                                  "#{build_year}": 0.0
                                                                },
                                                                "enfant_a_charge":{
                                                                  "#{build_year}": applicant_apl.valid_children_nb
                                                                }
                                                              }
        apl_body = {
          "individus": applicant,
          "menages": {
            "menage_1": {
              "personne_de_reference": [
                "#{applicant_apl.first_name}"
              ],
              "loyer": {
                "#{build_month}": 500.0
              },
              "zone_apl": {
                "#{build_month}": 'zone_2'
              },
              "statut_occupation_logement":{
                "#{build_month}": "locataire_vide"
              },
              "coloc":{
                "#{build_month}": false  
              }
            }
          },
          "familles": {
            "famille_1": {
              "parents": [
                "#{applicant_apl.first_name}"
              ],
            "aide_logement_montant": {
            "#{build_month}": nil
            }, 
            "aide_logement_base_ressources": {
              "#{build_month}": nil
            },
            "en_couple": {
              "#{build_month}": false
            }
          }
          },
          "foyers_fiscaux": {
            "foyer_fiscal_1": {
              "declarants": "#{applicant_apl.first_name}"
              
            }
          }
        }
        return apl_body
      end 
      it 'builds zone apl' do 
        expect(score_apl.build_zone_apl).to eq("zone_2")
      end 

      it 'builds the apl body' do 
        expect(score_apl.build_apl_body).to eq(build_body)
      end 
    end 
  end 

  context 'change that require and new score calculation' do 

    let!(:entity) { create(:entity) }
    let!(:applicant) { create(:applicant, 
      amount_year_minus_two: '20000',
      monthly_incomes: {
        'Salaire': 2000
      },
      birth_date: '2000-01-01',
      entity: entity
    ) }
    let!(:hr) { create(
      :housing_request, 
      main_applicant: applicant, 
      is_active: true,
      seeked_departments: ['33'],
    ) }
    let!(:offer) { create(
      :housing_offer, 
      typology: 'T1', 
      department: '33',
      threshold_type: 'PLUS',
      threshold_subtype: 'Paris et communes limitrophes',
      min_revenues: 1500,
      offer_status: 'LOGEMENT PUBLIE',
      rent_with_charges: 500,
    ) }

    it 'is eligible and create job' do 
      expect(Hermes::HousingOffer.where(applicant.eligibility_query.json_to_h).pluck(:id).include?(offer.id)).to be_truthy
      expect(Hermes::HousingRequest.where(offer.eligibility_query.json_to_h).pluck(:id).include?(hr.id)).to be_truthy
      expect(Hermes::Score.where(applicant: applicant, housing_offer: offer, is_active: true).count).to eq(0)
      jobs = enqueued_jobs.select {|job| job[:job] == UpdateScoreFromApplicantJob}
      expect(jobs).to include({:job=>UpdateScoreFromApplicantJob, :args=>[applicant.id.to_s], :queue=>"x_hermes_score_applicants"})
      drain_all_jobs
      expect(Hermes::Score.where(applicant: applicant, housing_offer: offer, is_active: true).count).to eq(0)
    end
    
    it 'When request become eligible create score' do 
      drain_all_jobs
      
      hr.request_status = "ELIGIBLE"
      hr.save
      drain_all_jobs
      expect(Hermes::Score.where(applicant: applicant, housing_offer: offer, is_active: true).count).to eq(1)
    end

    describe 'recalculate' do 
      before :each do 
        hr.request_status = "ELIGIBLE"
        hr.save
        drain_all_jobs
      end

      it 'normal score' do
        score = Hermes::Score.where(applicant: applicant, housing_offer: offer, is_active: true).first
        expect(score.profile_epci_score).to eq(310)
      end

      it 'When hr rejected_in_cal_nb change' do 
        hr.rejected_in_cal_nb = 1
        hr.save
        drain_all_jobs
        score = Hermes::Score.where(applicant: applicant, housing_offer: offer, is_active: true).first
        expect(score.profile_epci_score).to eq(350)
      end

      it 'When hr rejected_after_cal_nb change' do 
        hr.rejected_after_cal_nb = 1
        hr.save
        drain_all_jobs
        score = Hermes::Score.where(applicant: applicant, housing_offer: offer, is_active: true).first
        expect(score.profile_epci_score).to eq(270)
      end

      it 'When hr motivation1 change' do 
        hr.motivation1 = '19'
        hr.save
        drain_all_jobs
        score = Hermes::Score.where(applicant: applicant, housing_offer: offer, is_active: true).first
        expect(score.profile_epci_score).to eq(380)
      end

      it 'When applicant birth_date change' do 
        applicant.birth_date = "1980-01-01"
        applicant.save
        drain_all_jobs
        score = Hermes::Score.where(applicant: applicant, housing_offer: offer, is_active: true).first
        expect(score.profile_epci_score).to eq(240)
      end

      it 'When applicant monthly_incomes change' do 
        applicant.monthly_incomes = {
        'Salaire': 1600
        }
        applicant.save
        drain_all_jobs
        score = Hermes::Score.where(applicant: applicant, housing_offer: offer, is_active: true).first
        expect(score.profile_epci_score).to eq(350)
      end

      it 'When offer rent_with_charges change' do 
        offer.rent_with_charges = 600
        offer.save
        drain_all_jobs
        score = Hermes::Score.where(applicant: applicant, housing_offer: offer, is_active: true).first
        expect(score.profile_epci_score).to eq(350)
      end

      it 'When entity auto_request_validation change' do 
        entity.auto_request_validation = false
        entity.save
        drain_all_jobs
        score = Hermes::Score.where(applicant: applicant, housing_offer: offer, is_active: true).first
        expect(score.profile_epci_score).to eq(70)
      end

      it 'When hr entity_validated_changed' do 
        entity.update(auto_request_validation: false)
        hr.update(entity_validated: true)
        drain_all_jobs
        score = Hermes::Score.where(applicant: applicant, housing_offer: offer, is_active: true).first
        expect(score.profile_epci_score).to eq(310)
      end
      
    end
  end
end
